package com.smppw.analysis.domain.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.analysis.domain.dao.info.InfoFactory;
import com.smppw.analysis.domain.service.BaseIndicatorServiceV2;
import com.smppw.analysis.domain.service.NavService;
import com.smppw.common.pojo.dto.DateValue;
import com.smppw.common.pojo.dto.calc.IndicatorCalcPropertyDto;
import com.smppw.common.pojo.dto.indicator.CalcMultipleSecMultipleTimeRangeIndicatorReq;
import com.smppw.common.pojo.dto.indicator.DateIntervalDto;
import com.smppw.common.pojo.enums.*;
import com.smppw.common.pojo.enums.strategy.IStrategy;
import com.smppw.constants.SecType;
import com.smppw.core.IndicatorService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class BaseIndicatorServiceV2Impl implements BaseIndicatorServiceV2 {
    private final NavService navService;
    private final InfoFactory infoFactory;

    public BaseIndicatorServiceV2Impl(NavService navService, InfoFactory infoFactory) {
        this.navService = navService;
        this.infoFactory = infoFactory;
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> calcMultipleSecMultipleTimeRangeIndicator(CalcMultipleSecMultipleTimeRangeIndicatorReq req) {
        List<String> mainSecIdList = req.getMainSecIdList();
        Map<String, String> secBenchmarkIdMap = req.getSecBenchmarkIdMap();
        List<String> indexIdList = req.getIndexIdList();
        Visibility visibility = req.getVisibility();
        RaiseType raiseType = req.getRaiseType();
        IStrategy strategy = req.getStrategy();
        NavType navType = req.getNavType();
        String riskOfFreeId = req.getRiskOfFreeId();
        List<String> benchmarkIdList = secBenchmarkIdMap.values().stream().distinct().collect(Collectors.toList());
        if (StrUtil.isNotEmpty(riskOfFreeId)) {
            benchmarkIdList.add(riskOfFreeId);
        }
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, curveType.getId(), strategy.getStrategyId(), navType, visibility, MapUtil.empty());
        Map<String, Frequency> secFreqMap = this.loadMultiSecFreq(mainSecIdList);
        return IndicatorService.getInstance().calcMultipleSecMultipleTimeRangeIndicator(req, allNavMap, secFreqMap);
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> getMultipleSecTrend(List<String> mainSecIdList, Map<String, String> secBenchmarkIdMap,
                                                                           List<String> indexIdList, DateIntervalDto dateIntervalDto, Frequency frequency,
                                                                           BigDecimal fixedIncome, BigDecimal initValue, RaiseType raiseType, IStrategy strategy,
                                                                           Visibility visibility, NavType navType, List<TrendType> trendTypeV2List) {
        List<String> benchmarkIdList = secBenchmarkIdMap.values().stream().distinct().collect(Collectors.toList());
        CurveType curveType = CurveType.getCurveType(raiseType, strategy);
        Map<String, List<DateValue>> allNavMap = navService.getSecIdDateValueNavListMapFromRedisAndDB(mainSecIdList, benchmarkIdList, indexIdList,
                null, null, curveType.getId(), strategy.getStrategyId(), navType, visibility, MapUtil.empty());
        Map<String, Frequency> secFreqMap = this.loadMultiSecFreq(mainSecIdList);
        return IndicatorService.getInstance().getMultipleSecTrend(mainSecIdList, secBenchmarkIdMap, indexIdList, dateIntervalDto, fixedIncome,
                initValue, frequency, trendTypeV2List, secFreqMap, allNavMap);
    }

    @Override
    public Map<String, List<IndicatorCalcPropertyDto>> getMultiSecRetListNew(List<String> mainSecIdList, List<String> indexIds, Frequency frequency, Frequency rollingFrequency,
                                                                             String startDate, String endDate, boolean ifExcessReturn, String benchmarkId, RaiseType raiseType, IStrategy strategy,
                                                                             Visibility visible, NavType navType, Map<String, List<DateValue>> allNavMap) {
        Map<String, Frequency> secFreqMap = this.loadMultiSecFreq(mainSecIdList);
        return IndicatorService.getInstance().getMultiSecRetListNew(mainSecIdList, indexIds, frequency, rollingFrequency, startDate, endDate, ifExcessReturn,
                benchmarkId, raiseType, strategy, visible, navType, allNavMap, secFreqMap);
    }

    /**
     * 获取多种类型的标的的净值频率
     *
     * @param secIds 多个标的id集合
     * @return /
     */
    private Map<String, Frequency> loadMultiSecFreq(List<String> secIds) {
        if (CollUtil.isEmpty(secIds)) {
            return MapUtil.empty();
        }
        Map<String, List<String>> typeSecMap = SecType.getTypeSecMap(secIds, null);
        Map<String, Frequency> temp = MapUtil.newHashMap(secIds.size());
        typeSecMap.forEach((k, v) -> {
            Map<String, Frequency> secNameMap = this.infoFactory.getInstance(k).getSecFreqMap(v);
            temp.putAll(secNameMap);
        });
        Map<String, Frequency> result = MapUtil.newHashMap(secIds.size());
        for (String secId : secIds) {
            result.put(secId, temp.get(secId));
        }
        return result;
    }
}
